@@ -59,11 +59,13 @@ import javax.swing.JScrollPane;
 
 import org.eclipse.jgit.awtui.CommitGraphPane;
 import org.eclipse.jgit.lib.Constants;
+import org.eclipse.jgit.pgm.internal.CLIText;
 import org.eclipse.jgit.revplot.PlotWalk;
 import org.eclipse.jgit.revwalk.RevCommit;
 import org.eclipse.jgit.revwalk.RevSort;
 import org.eclipse.jgit.revwalk.RevWalk;
 
+@Command(usage = "usage_Glog")
 class Glog extends RevWalkTextBuiltin {
 	final JFrame frame;
 
@@ -86,6 +88,7 @@ class Glog extends RevWalkTextBuiltin {
 		final JButton repaint = new JButton();
 		repaint.setText(CLIText.get().repaint);
 		repaint.addActionListener(new ActionListener() {
+			@Override
 			public void actionPerformed(ActionEvent e) {
 				graphPane.repaint();
 			}
@@ -99,6 +102,7 @@ class Glog extends RevWalkTextBuiltin {
 		frame.getContentPane().add(world);
 	}
 
+	/** {@inheritDoc} */
 	@Override
 	protected int walkLoop() throws Exception {
 		graphPane.getCommitList().source(walk);
@@ -110,11 +114,13 @@ class Glog extends RevWalkTextBuiltin {
 		return graphPane.getCommitList().size();
 	}
 
+	/** {@inheritDoc} */
 	@Override
 	protected void show(final RevCommit c) throws Exception {
 		throw new UnsupportedOperationException();
 	}
 
+	/** {@inheritDoc} */
 	@Override
 	protected RevWalk createWalk() {
 		if (objects)
